-- Puffin: A Tojaso FowlPlay Project
-- Adapter component for assembling data brokers and updating them as needed.
-- Brokers are displayed in blocks that are assembled from configuration layout records.
-- Placement is based on named anchor points (eventually these will be moveable).
-- This component must avoid configuring while in global lockdown.

local Adapter_Brokers = {}
local EF = Puffin.ErrorFormat
local DB, UN, BB, DP, LSM
local active_brokers = {}
local active_offsetX = 0
local active_offsetY = 0
local broker_count = 0
local broker_changed = {}
local broker_created = {}
local configuration_changed = false

local function BrokerChanged(event, name, key) broker_changed[name] = true end
local function BrokerCreated(event, name) broker_created[name] = true; configuration_changed = true end

function Adapter_Brokers.OnInitialize()	
	Puffin.RegisterCallback("Broker_Changed", BrokerChanged)
	Puffin.RegisterCallback("Broker_Created", BrokerCreated)
end

function Adapter_Brokers.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, EF("Adapter_Brokers", "Data component not found"))
	BB = Puffin.Component("View_BrokerBlock")
	assert(BB, EF("Adapter_Brokers", "View_BrokerBlock component not found"))
	LSM = LibStub("LibSharedMedia-3.0")
	assert(LSM, EF("Adapter_Brokers", "LibSharedMedia not found"))
end

-- On first update, configure the brokers, afterwards just update each broker as needed
function Adapter_Brokers.OnUpdate()
	local v = DB.GetRecord("layouts", "brokers")
	if not v then return end
	local bb = v.brokers
	if not bb then return end
	if not Puffin.inLockdown and configuration_changed then -- can't configure while in combat lockdown
		for order = 1, 20 do -- maximum number of broker groups
			for name, isCreated in pairs(broker_created) do -- only configure brokers once, they will never be released
				if isCreated then
					local b = bb[name]
					if b and b.order == order then -- create them in the correct order
						broker_created[name] = false
						local w = (b.icon and v.iconSize or 0) + (b.text and b.textWidth or 0)
						local u = BB.Create() -- create the broker block view
						u.defaults = v.defaults; u.depth = v.depth; u.width = w; u.height = v.iconSize; u.base = v.base
						u.icon = b.icon and v.icon or nil; u.text = b.text and v.text or nil
						local dx, dy = v.dirX * (u.width + v.spaceX), v.dirY * (u.height + v.spaceY)
						u.offsetX = v.anchor.x + dx + active_offsetX; u.offsetY = v.anchor.y + dy + active_offsetY
						u.anchorPoint = v.anchor.anchorPoint; u.relativePoint = v.anchor.relativePoint
						BB.Configure(u)
						active_offsetX = active_offsetX + dx; active_offsetY = active_offsetY + dy
						active_brokers[name] = u
						broker_count = broker_count + 1
						broker_changed[name] = true
					end
				end
			end
		end
		configuration_changed = false -- all created data brokers have been processed
	end

	for name, isChanged in pairs(broker_changed) do -- update all the changed broker blocks
		if isChanged then
			local broker = DB.GetAttribute("brokers", name, "broker")
			local u = active_brokers[name]
			if u and broker then BB.Update(u, broker) end
		end
		broker_changed[name] = false
	end
end

Puffin.RegisterComponent("Adapter_Brokers", Adapter_Brokers, "ADAPTER", 2)
