-- Puffin: A Tojaso FowlPlay Project
-- Adapter component for configuration data

local Adapter_Configuration = {}
local EF = Puffin.ErrorFormat
local DB
local defaultWhite = { r = 1, g = 1, b = 1, a = 1 }
local defaultBlack = { r = 0, g = 0, b = 0, a = 1 }
local defaultGray = { r = 0.25, g = 0.25, b = 0.25, a = 1 }
local defaultLight = { r = 0.75, g = 0.75, b = 0.75, a = 1 }
local defaultYellow = { r = 1, g = 1, b = 0, a = 1 }
local defaultTransparentBlack = { r = 0, g = 0, b = 0, a = 0.5 }
local defaultTransparentGray = { r = 0.25, g = 0.25, b = 0.25, a = 0.5 }
local mana_color = { r = 0.447, g = 0.62353, b = 0.8118, a = 1 }
local mouseover_color = { r = 1, g = 1, b = 0, a = 0.5 }

-- Defaults applied to all instances of each widget class
local defaults = {
	Widget_Text = {
		font = { "#LSM", "font", "Arial Narrow" }, -- font to use for the text (default: ChatFontNormal) [c]
		fsize = 10, -- font size for the text (default: 10) [c]
		shadow = true, -- enable a shadow behind the text (default: false) [c]
		outline = false, -- enable font "OUTLINE" flag (default: false) [c]
		thick = false, -- enable font "THICKOUTLINE" flag (default: false) [c]
		mono = false, -- enable font "MONOCHROME" flag (default: false) [c]
		wrap = false, -- enable wrap of text to multiple lines (default: false) [c]
		justifyH = "CENTER", -- horizontal text alignment (default: "CENTER") [c]
		justifyV = "MIDDLE", -- vertical text alignment (default: "MIDDLE") [c]
	},
	Widget_Icon = {
	},
	Widget_Rectangle = {
		texture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar",
		borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare3", -- if defined then show a border around the filled area of the rectangle [d]
		borderWidth = 4, -- width of border (default: 8) [d]
		borderOffset = 1, -- offset of border from texture (default: 2) [d]
	},
	Widget_Panel = {
		borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare3", -- if defined then show a border around the base [c]
		borderWidth = 4, -- width of border (default: 8) [c]
		borderOffset = 1, -- offset of border from texture (default: 2) [c]
		panelTexture = { "#LSM", "background", "Blizzard Tooltip" }, -- if defined then fill the base with colored texture [c]
		panelInset = 1, -- inset of base texture from border (default: 0) [c]
		panelColor = defaultGray,
	},
	Widget_Bar = {
		fgTexture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar",
		bgTexture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar", -- texture to use for the bar background (default: same as foreground) [c]
		direction = false, -- which way foreground bar moves as value decreases, false=right-to-left, true=left-to-right (default: false) [c]
		sparkAlpha = 0.5, -- opacity of spark displayed at moving end of the foreground bar (default: 0) [c]
	},
}

local misc_templates = {
	panel = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, relativeHeight = 1, depth = 0,
		attributes = { panelColor = defaultTransparentGray, borderColor = defaultTransparentBlack, },
	},
}

local standard_templates = {
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 0, alpha = 0.4,
		attributes = { borderOffsetX = -1, borderOffsetY = -1, panelInset = 1.5 },
		providers = {
			alpha = { class = "unit_number", name = "range_alpha" },
			borderColor = { class = "unit_color", name = "mouseover" },
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		offsetX = 1, offsetY = 1, relativeY = 0.4375, relativeWidth = 1, width = -2, relativeHeight = 0.5625, height = -2, depth = 2,
		providers = {
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
			sparkAlpha = { class = "unit_number", name = "dead_spark_alpha" },
		},
	},
	powerBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT", 
		offsetX = 1, relativeY = 0.3125, relativeWidth = 1, width = -2, relativeHeight = 0.125, depth = 2,
		providers = {
			fgColor = { class = "unit_color", name = "power" },
			bgColor = { class = "unit_color", name = "power_background" },
			value = { class = "unit_number", name = "power_fraction" },
			sparkAlpha = { class = "unit_number", name = "dead_spark_alpha" },
		},
	},
	aggroBar = {
		priority = 2, class = "Widget_Rectangle", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		relativeWidth = 1, relativeHeight = 0.3125, depth = 4,
		attributes = {
			valueWidth = 1, valueHeight = 1, valueOffsetX = 0, valueAlpha = 0, borderOffset = 0, alpha = 0.8,
		},
		providers = {
			condition = { class = "unit_condition", name = "aggro" },
			borderColor = { class = "unit_color", name = "threat" },
		},
	},
	incomingBar = {
		priority = 3, class = "Widget_Rectangle", parent = "base", anchor = "healthBar", anchorRegion = "fgTexture", anchorPoint = "LEFT", relativePoint = "RIGHT",
		relativeWidth = 1, relativeHeight = 0.4, depth = 4, alpha = 0.5,
		attributes = { minimumWidth = 4, valueHeight = 1, },
		providers = {
			color = { class = "unit_color", name = "health" },
			valueWidth = { class = "unit_number", name = "incoming_fraction" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "LEFT", relativePoint = "LEFT",
		offsetX = 3, relativeWidth = 0.7, relativeHeight = 0.5, depth = 6,
		attributes = { fsize = 10, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultWhite  },
		providers = {
			text = { class = "unit_text", name = "name" },
			color = { class = "unit_color", name = "class" },
		},
	},
	classText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM",
		offsetX = 3, relativeWidth = 1, relativeHeight = 0.3125, depth = 6,
		attributes = { fsize = 7, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultWhite  },
		providers = {
			text = { class = "unit_text", name = "level_class" },
		},
	},
	healthText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "RIGHT", relativePoint = "RIGHT",
		offsetX = -2, relativeWidth = 0.3, relativeHeight = 0.5, depth = 6,
		attributes = { fsize = 7, justifyH = "RIGHT", justifyV = "MIDDLE", color = defaultLight, },
		providers = {
			text = { class = "unit_text", name = "health_percent" },
			-- color = { class = "unit_color", name = "health" },
		},
	},
	powerText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT",
		offsetX = -4, relativeWidth = 0.3, relativeHeight = 0.3125, depth = 6,
		attributes = { fsize = 7, justifyH = "RIGHT", justifyV = "MIDDLE", color = defaultLight, },
		providers = {
			text = { class = "unit_text", name = "power_status" },
		},
	},
	statusText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		offsetX = 3, relativeWidth = 0.25, relativeHeight = 0.3125, depth = 6,
		attributes = { fsize = 7, justifyH = "LEFT", justifyV = "MIDDLE", },
		providers = {
			text = { class = "unit_text", name = "status" },
			color = { class = "unit_color", name = "status" },
		},
	},
	combatIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM",
		width = 12, height = 12, depth = 8, offsetY = -1,
		attributes = { texture = [[Interface\CharacterFrame\UI-StateIcon]], texcoord = { .5, 1, 0, .49 }, endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "in_combat" },
		},
	},
	pvpIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPRIGHT",
		width = 12, height = 12, depth = 8, offsetX = -2, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "is_pvp" },
			texture = { class = "unit_icon", name = "pvp" },
		},
	},
	pvpText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "pvpIcon", anchorPoint = "RIGHT", relativePoint = "LEFT",
		relativeWidth = 0.25, height = 8, depth = 8, offsetY = 4,
		attributes = { fsize = 7, justifyH = "RIGHT" },
		providers = {
			text = { class = "unit_text", name = "pvp_timer" },
			color = { class = "color_palette", name = "red" },
		},
	},
	comboText = {
		priority = 2, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "TOP",
		relativeWidth = 0.5, height = 10, depth = 6,
		attributes = { fsize = 10, justifyH = "CENTER", justifyV = "BOTTOM", },
		providers = {
			text = { class = "unit_text", name = "combo_points" },
			condition = { class = "unit_condition", name = "combo_points" },
		},
	},
	questIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "quest_boss" },
			texture = { class = "unit_icon", name = "quest_icon" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP", 
		width = 12, height = 12, scale = 0.75, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
	leaderIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "BOTTOMLEFT", 
		width = 12, height = 12, scale = 0.5, depth = 8, offsetX = 1, offsetY = -1,
		providers = {
			condition = { class = "unit_condition", name = "is_leader" },
			texture = { class = "unit_icon", name = "leader_icon" },
		},
	},
	looterIcon = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "leaderIcon", anchorRegion = "texture", anchorPoint = "TOPLEFT", relativePoint = "TOPRIGHT",
		width = 12, height = 12, scale = 0.5, depth = 8, offsetX = 2,
		providers = {
			condition = { class = "unit_condition", name = "is_master_looter" },
			texture = { class = "unit_icon", name = "master_looter_icon" },
		},
	},
	roleIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "BOTTOMRIGHT",
		width = 12, height = 12, scale = 0.75, alpha = 0.8, offsetX = -2, offsetY = -1, depth = 8,
		providers = {
			condition = { class = "unit_condition", name = "assigned_role" },
			texture = { class = "unit_icon", name = "role_icon" },
		},
	},
	restIcon = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPLEFT",
		width = 12, height = 12, depth = 8, offsetX = 2, offsetY = -2,
		attributes = { endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "is_resting" },
			texture = { class = "unit_icon", name = "rest_icon" },
		},
	},
}

local raid_templates = {
	backdrop = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 8, relativeHeight = 1, height = 8, depth = 0,
		attributes = { panelColor = defaultTransparentGray, borderColor = defaultTransparentBlack, },
	},
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 0, alpha = 0.4,
		attributes = { borderOffsetX = -1, borderOffsetY = -1, panelInset = 1.5 },
		providers = {
			alpha = { class = "unit_number", name = "range_alpha" },
			borderColor = { class = "unit_color", name = "mouseover" },
		},
	},
	aggroBar = {
		priority = 2, class = "Widget_Rectangle", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 4,
		attributes = {
			valueWidth = 1, valueHeight = 1, valueAlpha = 0, alpha = 0.8, borderOffsetX = -1, borderOffsetY = -1,
		},
		providers = {
			condition = { class = "unit_condition", name = "aggro" },
			borderColor = { class = "unit_color", name = "threat" },		
		},
	},
	debuffBar = {
		priority = 2, class = "Widget_Rectangle", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		relativeWidth = 1, relativeHeight = 1, offsetY = -1, depth = 4,
		attributes = {
			valueWidth = 1, valueHeight = 1, valueAlpha = 0, alpha = 0.8, borderOffsetX = 0, borderOffsetY = 1,
		},
		providers = {
			condition = { class = "unit_condition", name = "debuff" },
			borderColor = { class = "unit_color", name = "debuff" },		
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		offsetX = 1, relativeY = 0.2, offsetY = 1, relativeWidth = 1, width = -2, relativeHeight = 0.8, height = -2, depth = 2,
		attributes = { sparkAlpha = 0, },
		providers = {
			-- condition = { class = "unit_condition", name = "power_mana" },
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
		},
	},
	powerBar = {
		priority = 3, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		offsetX = 1, offsetY = 1, relativeWidth = 1, width = -2, relativeHeight = 0.2, height = -1, depth = 2,
		attributes = { sparkAlpha = 0, },
		providers = {
			-- condition = { class = "unit_condition", name = "power_mana" },
			fgColor = { class = "unit_color", name = "power" },
			bgColor = { class = "unit_color", name = "power_background" },
			value = { class = "unit_number", name = "power_fraction" },
		},
	},
	incomingBar = {
		priority = 3, class = "Widget_Rectangle", parent = "base", anchor = "healthBar", anchorRegion = "fgTexture", anchorPoint = "LEFT", relativePoint = "RIGHT",
		relativeWidth = 1, relativeHeight = 0.6, depth = 4, alpha = 0.5, offsetX = 0.5,
		attributes = { minimumWidth = 2, valueHeight = 1, },
		providers = {
			-- condition = { class = "unit_condition", name = "power_mana" },
			color = { class = "unit_color", name = "health" },
			valueWidth = { class = "unit_number", name = "incoming_fraction" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "CENTER", relativePoint = "CENTER",
		offsetX = 3, relativeWidth = 0.95, relativeHeight = 0.95, depth = 6,
		attributes = { fsize = 9, justifyH = "LEFT", justifyV = "MIDDLE", },
		providers = {
			text = { class = "unit_text", name = "name" },
			color = { class = "unit_color", name = "class" },
		},
	},
	statusText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "powerBar", anchorPoint = "CENTER", relativePoint = "CENTER",
		relativeWidth = 1, relativeHeight = 0.5, depth = 6,
		attributes = { fsize = 7, justifyH = "CENTER", justifyV = "MIDDLE", },
		providers = {
			text = { class = "unit_text", name = "status" },
			color = { class = "unit_color", name = "status" },
		},
	},
	readyIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 16, height = 16, depth = 8,
		attributes = { endAnimation = "delay" },
		providers = {
			condition = { class = "unit_condition", name = "ready_check_active" },
			texture = { class = "unit_icon", name = "ready_icon" },
		},
	},
	arrowIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 24, height = 24, depth = 8,
		attributes = { endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "show_direction" },
			texture = { class = "unit_icon", name = "arrow_icon" },
			rotation = { class = "unit_number", name = "direction" },
			color = { class = "color_palette", name = "yellow" },
		},
	},
	pvpIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPRIGHT",
		width = 12, height = 12, scale = 0.75, depth = 8, offsetX = -2, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "is_pvp" },
			texture = { class = "unit_icon", name = "pvp" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, scale = 0.65, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
	icon1 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPLEFT", relativePoint = "TOPLEFT",
		width = 10, height = 10, depth = 8, offsetX = 2, offsetY = -4, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition1" },
			color = { class = "unit_color", name = "class_color1" },
		},
	},
	text1 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon1", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite },
		providers = {
			condition = { class = "unit_condition", name = "class_condition1" },
			text = { class = "unit_text", name = "class_text1" },
		},
	},
	icon2 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		width = 10, height = 10, depth = 8, offsetX = 2, offsetY = 2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition2" },
			color = { class = "unit_color", name = "class_color2" },
		},
	},
	text2 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon2", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite },
		providers = {
			condition = { class = "unit_condition", name = "class_condition2" },
			text = { class = "unit_text", name = "class_text2" },
		},
	},
	icon3 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT",
		width = 10, height = 10, depth = 8, offsetX = -4, offsetY = -4, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition3" },
			color = { class = "unit_color", name = "class_color3" },
		},
	},
	text3 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon3", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite },
		providers = {
			condition = { class = "unit_condition", name = "class_condition3" },
			text = { class = "unit_text", name = "class_text3" },
		},
	},
	icon4 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT",
		width = 10, height = 10, depth = 8, offsetX = -4, offsetY = 2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition4" },
			color = { class = "unit_color", name = "class_color4" },
		},
	},
	text4 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon4", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite },
		providers = {
			condition = { class = "unit_condition", name = "class_condition4" },
			text = { class = "unit_text", name = "class_text4" },
		},
	},
	icon5 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM",
		width = 10, height = 10, depth = 8, offsetY = 2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition5" },
			color = { class = "unit_color", name = "class_color5" },
		},
	},
	icon6 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPLEFT", relativePoint = "TOPLEFT",
		width = 8, height = 8, depth = 8, offsetX = 12, scale = 0.75,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Circle", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition6" },
			color = { class = "unit_color", name = "class_color6" },
		},
	},
	icon7 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT",
		width = 8, height = 8, depth = 8, offsetX = -12, scale = 0.75,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Circle", },
		-- attributes = { texture = "Interface\\Icons\\Spell_Nature_Regeneration", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition7" },
			color = { class = "unit_color", name = "class_color7" },
		},
	},
}

local raid_target_templates = {
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 0, alpha = 0.5,
		attributes = { borderOffsetX = -1, borderOffsetY = -1 },
		providers = {
			borderColor = { class = "unit_color", name = "mouseover" },
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		offsetX = 1, offsetY = 1, relativeWidth = 1, width = -2, relativeHeight = 1, height = -2, depth = 2,
		attributes = { sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "CENTER",relativePoint = "CENTER",
		offsetX = 3, offsetY = -1, relativeWidth = 0.95, relativeHeight = 0.95, depth = 6,
		attributes = { fsize = 7, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultWhite  },
		providers = {
			text = { class = "unit_text", name = "name" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, scale = 0.65, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
}

local playerWidgets = {
	"base", "healthBar", "powerBar", "aggroBar", "incomingBar", "nameText", "healthText", "powerText", "classText", "pvpText",
	"combatIcon", "restIcon", "pvpIcon", "raidIcon", "roleIcon", "leaderIcon", "looterIcon",
}

local normalWidgets = {
	"base", "healthBar", "powerBar", "aggroBar", "incomingBar", "nameText", "healthText", "powerText", "classText", "comboText",
	"pvpIcon", "raidIcon", "roleIcon", "leaderIcon", "looterIcon", 
}

local smallWidgets = {
	"base", "healthBar", "powerBar", "aggroBar", "incomingBar", "nameText", "classText", "pvpIcon", "raidIcon",
}

local raidWidgets = {
	"base", "healthBar", "powerBar", "aggroBar", "incomingBar", "nameText", "statusText", "readyIcon", "pvpIcon", "raidIcon", "arrowIcon",
	"icon1", "icon2", "icon3", "icon4", "icon5", "icon6", "icon7", "text1", "text2", "text3", "text4",
}
local raidSmallWidgets = {
	"base", "healthBar", "powerBar", "aggroBar", "incomingBar", "nameText", "statusText", "raidIcon",
}

local raidTargetWidgets = {
	"base", "healthBar", "nameText", "raidIcon",
}

local backdropWidgets = { "backdrop", }
local panelWidgets = { "panel", }

local layout = {
	defaults = "standard", templates = "standard",
	anchors = {
		left_anchor = { x = 0, y = 0, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
		bottom_anchor = { x = 0, y = 0, anchorPoint = "BOTTOM", relativePoint = "BOTTOM" },
		right_anchor = { x = 0, y = 0, anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT" },
		solo_anchor = { x = 0, y = 92, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOM" },
		party_anchor = { x = 20, y = 500, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
		raid_anchor = { x = 20, y = 250, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
		heal_anchor = { x = 490, y = 200, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
	},
	panels = {	
		bottom_bar = { anchor = "bottom_anchor", templates = "misc", widgets = "panel", width = 324, height = 30, offsetX = 0, offsetY = 6, depth = 0, },
		left_stripe = { anchor = "left_anchor", templates = "misc", widgets = "panel", width = 470, height = 4, offsetX = 8, offsetY = 10, depth = 0, },
		right_stripe = { anchor = "right_anchor", templates = "misc", widgets = "panel", width = 470, height = 4, offsetX = -8, offsetY = 10, depth = 0, },
	},
	units = {	
		player = { anchor = "solo_anchor", widgets = "player", width = 150, height = 32, offsetX = -160, offsetY = 27, depth = 1, clique = true },
		pet = { anchor = "solo_anchor", widgets = "small", width = 120, height = 32, offsetX = -287, offsetY = -36, depth = 1, },
		pettarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 22, offsetX = -242, offsetY = -63, depth = 1, },
		target = { anchor = "solo_anchor", widgets = "normal", width = 150, height = 32, offsetX = 10, offsetY = 27, depth = 1, },
		targettarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 22, offsetX = 10, offsetY = 0, depth = 1, },
		focus = { anchor = "solo_anchor", widgets = "small", width = 120, height = 32, offsetX = 167, offsetY = -36, depth = 1, },
		focustarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 22, offsetX = 167, offsetY = -63, depth = 1, },
	},
	party = {
		anchor = "raid_anchor", heal_anchor = "heal_anchor", templates = "raid", depth = 1, offsetX = 63, offsetY = 0, player = true,
		party = { widgets = "raid", width = 60, height = 22, offsetX = 0, offsetY = 12, clique = true },
		target = { templates = "raid_target", widgets = "raid_target", width = 60, height = 9, offsetX = 0, offsetY = 0 },
		backdrop = { widgets = "backdrop", width = 310, height = 32, offsetX = -1.5, offsetY = -1.5, depth = 0, },
	},
	raid = {
		anchor = "raid_anchor", heal_anchor = "heal_anchor", templates = "raid", depth = 1,
		offsetX = 63, offsetY = 0, groupOffsetX = 0, groupOffsetY = 26, rowPoint = "LEFT", columnPoint = "TOP",
		player = { widgets = "raid", width = 60, height = 22, offsetX = 0, offsetY = 0, clique = true },
		backdrop = { widgets = "backdrop", width = 310, height = 1, offsetX = -1.5, offsetY = -2, depth = 0, },
	},
}

-- Add default and layout configuration data to the database
function Adapter_Configuration.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, EF("Adapter_Configuration", "Data component not found"))
	
	local class = Puffin.myClass -- check if using heal anchors
	local tree = nil
	local index = GetPrimaryTalentTree()
	if (class == "DRUID" and index == 3) or (class == "PRIEST" and index == 2) then
		Puffin.myRole = "healer"
		layout.party.anchor = layout.party.heal_anchor; layout.raid.anchor = layout.raid.heal_anchor
	end
	
	DB.AllocateDatabase("defaults", false)
	DB.SetAttribute("defaults", "colors", "mana", mana_color)
	DB.SetAttribute("defaults", "colors", "level", defaultYellow)
	DB.SetAttribute("defaults", "colors", "race", defaultLight)
	DB.SetAttribute("defaults", "colors", "mouseover", mouseover_color)
	DB.SetAttribute("defaults", "number", "raid_groups_max", 2)
	DB.SetAttribute("defaults", "number", "out_of_range_alpha", 0.5)
	
	DB.AllocateDatabase("widget_defaults", false)
	DB.SetRecord("widget_defaults", "standard", defaults)

	DB.AllocateDatabase("widget_templates", false)
	DB.SetRecord("widget_templates", "standard", standard_templates) 
	DB.SetRecord("widget_templates", "raid", raid_templates)
	DB.SetRecord("widget_templates", "raid_target", raid_target_templates)
	DB.SetRecord("widget_templates", "misc", misc_templates) 

	DB.AllocateDatabase("widgets", false)
	DB.SetRecord("widgets", "player", playerWidgets)
	DB.SetRecord("widgets", "normal", normalWidgets)
	DB.SetRecord("widgets", "small", smallWidgets)
	DB.SetRecord("widgets", "raid", raidWidgets)
	DB.SetRecord("widgets", "raid_small", raidSmallWidgets)
	DB.SetRecord("widgets", "raid_target", raidTargetWidgets)
	DB.SetRecord("widgets", "backdrop", backdropWidgets)
	DB.SetRecord("widgets", "panel", panelWidgets)

	DB.AllocateDatabase("layouts", false)
	DB.SetRecord("layouts", "standard", layout)
end

Puffin.RegisterComponent("Adapter_Configuration", Adapter_Configuration, "ADAPTER", 0)
