-- Puffin: A Tojaso FowlPlay Project
-- Model component for tracking aura information on all units
-- Not implemented: weapon enchants

local Model_Auras = {}
local DB = nil
local recycled = {}
local aura_changed = {}

local function UnitAuraEvent(e, unit) aura_changed[unit] = true end

function Model_Auras.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, "Puffin.Model_Auras: invalid reference to data management component")
	DB.AllocateDatabase("auras", false)
	Puffin.RegisterCallback("UNIT_AURA", UnitAuraEvent) -- register for unit aura events
end

-- Allocate aura tables, use recycling to minimize garbage generation
function Model_Auras.Allocate()
	local n, t = next(recycled) -- get any available recycled aura table
	if n then recycled[n] = nil else
		t = Puffin.AllocateTable(); t.buffs = {}; t.debuffs = {}; t.buffs.cache = {}; t.buffs.atypes = {}; t.debuffs.cache = {}; t.debuffs.atypes = {}
		for k = 1, 40 do t.buffs[k] = {}; t.debuffs[k] = {} end
	end
	t.buffs.count = 0; t.debuffs.count = 0
	return t
end

-- Release and recycle aura tables
function Model_Auras.Release(t) table.insert(recycled, t); return nil end

-- Get current auras for a unit and store them in the table
function Model_Auras.Auras(unit, t)
	local i, now = 0, GetTime(); table.wipe(t.buffs.cache); table.wipe(t.buffs.atypes)
	repeat
		local b = t.buffs[i + 1]
		b.name, b.rank, b.icon, b.count, b.atype, b.duration, b.expire, b.caster, b.steal, b.cons, b.id, b.apply, b.boss = UnitAura(unit, i + 1, "HELPFUL")
		if b.name then
			if b.caster then b.casterName = UnitName(b.caster); b.casterPlayer = UnitIsPlayer(b.caster); b.casterVehicle = UnitHasVehicleUI(b.caster)
			else b.casterName = nil; b.casterPlayer = nil; b.casterVehicle = nil end
			if (b.duration > 0) and (b.expire > now) then b.timeLeft = b.expire - now else b.timeLeft = 0 end
			if b.atype then t.buffs.atypes[b.atype] = true end
			t.buffs.cache[b.name] = true; i = i + 1
		end
	until not b.name or (i == 40)
	t.buffs.count = i
	i = 0; table.wipe(t.debuffs.cache); table.wipe(t.debuffs.atypes)
	repeat
		local b = t.debuffs[i + 1]
		b.name, b.rank, b.icon, b.count, b.atype, b.duration, b.expire, b.caster, b.steal, b.cons, b.id, b.apply, b.boss = UnitAura(unit, i + 1, "HARMFUL")
		if b.name then
			if b.caster then b.casterName = UnitName(b.caster); b.casterPlayer = UnitIsPlayer(b.caster); b.casterVehicle = UnitHasVehicleUI(b.caster)
			else b.casterName = nil; b.casterPlayer = nil; b.casterVehicle = nil end
			if (b.duration > 0) and (b.expire > now) then b.timeLeft = b.expire - now else b.timeLeft = 0 end
			if b.atype then t.debuffs.atypes[b.atype] = true end
			t.debuffs.cache[b.name] = true; i = i + 1
		end
	until not b.name or (i == 40)
	t.debuffs.count = i
	local g = UnitGUID(unit)
	for k in pairs(aura_changed) do if g == UnitGUID(k) then return true end end
	return false
end

-- Update is called after that for the generic unit model
function Model_Auras.OnUpdate() table.wipe(aura_changed) end -- initialize change table so tracking changes since this update

Puffin.RegisterComponent("Model_Auras", Model_Auras, "MODEL", 3)
