-- Puffin: A Tojaso FowlPlay Project
-- Invisible widget to handle interactive input as a secure action button

-- Custom attributes:

local Widget_Button = {}
local EF = Puffin.ErrorFormat
local recycledButton = {}
local buttonCount = 0

-- Create a new button widget object
function Widget_Button.Create()
	local n, b = next(recycledButton) -- get any available recycled bar
	if n then
		recycledButton[n] = nil
	else
		b = {}
		buttonCount = buttonCount + 1
		b._display = {} -- table of display objects preserved when widget is recycled
		local frameName = "Puffin_Button" .. tostring(buttonCount) -- generate unique name for container frame
		b._display.container = CreateFrame("Button", frameName .. "Container", UIParent, "SecureActionButtonTemplate")
		b._display.container:SetFrameStrata("MEDIUM")
	end
	return b
end

-- Release when no longer needed, recycle object after clearing attributes and hiding display objects associated with it
function Widget_Button:Release()
	for k in pairs(self) do if k ~= "_display" then self[k] = nil end end
	table.insert(recycledButton, self)
end

-- Set display position for the widget relative to another widget or the overall screen
function Widget_Button:SetPosition(anchor, anchorWidget, relative, offsetX, offsetY)
	local t, w = self._display, UIParent
	if anchorWidget and anchorWidget._display then w = anchorWidget._display.container end
	t.container:ClearAllPoints(); t.container:SetPoint(anchor, w, relative, offsetX, offsetY)
end

-- Set the parent widget (used for managing display heirarchy)
function Widget_Button:SetParent(widget)
	local t, w = self._display, widget._display
	if w and w.container then w = w.container else w = UIParent end
	t.container:SetParent(w)
	if not self.depth then t.container:SetFrameLevel(w:GetFrameLevel() + 1) end
end

-- Show tooltip when entering a unit frame
local function Button_OnEnter(b)
	if GetCVar("UberTooltips") == "1" then
		GameTooltip_SetDefaultAnchor(GameTooltip, b)
	else
		GameTooltip:SetOwner(b, "BOTTOMRIGHT")
	end
end

-- Hide tooltip when leaving a unit frame
local function Button_OnLeave() GameTooltip:Hide() end

-- Update attributes associated with the secure action button
function Widget_Button:Configure()
	if InCombatLockdown() then return end
	assert(self.width and self.width > 0, EF("Widget_Button", "undefined width"))
	assert(self.height and self.height > 0, EF("Widget_Button", "undefined height"))
	
	local b = self._display.container
	local d = self.depth; if d then b:SetFrameLevel(d) end
	b:SetSize(self.width, self.height); b:SetScale(self.scale or 1)
	-- local tex = b:CreateTexture(); tex:SetAllPoints(); tex:SetTexture(1,0,0); tex:SetAlpha(0.25) -- debugging red overlay
	if self.unit then
		b:SetScript("OnEnter", Button_OnEnter)
		b:SetScript("OnLeave", Button_OnLeave)
		b:EnableMouse(true)
		b:RegisterForClicks("AnyUp")
	end
end

-- Null update function for unit frame widget
function Widget_Button:Update() end

-- Register this widget as a component so it can be found by views
Puffin.RegisterComponent("Widget_Button", Widget_Button, "WIDGET", 0)
